import jarvis
from discord import File
import cv2
import aiohttp
import re
import numpy as np
from jarvis.utils import convert_bytesize, unconvert_bytesize
from discord.ext import commands
from io import BytesIO


class ImageCog(commands.Cog):
    """
    Image processing functions for J.A.R.V.I.S.

    May be categorized under util later
    """

    def __init__(self, bot):
        self.bot = bot
        self._session = aiohttp.ClientSession()
        self.tgt_match = re.compile(
            r"([0-9]*\.?[0-9]*?) ?([KMGTP]?B)", re.IGNORECASE
        )

    @commands.command(name="resize", help="Resize an image")
    async def _resize(self, ctx, target: str, url: str = None):
        if not target:
            await ctx.send("Missing target size, i.e. 200KB.")
            return

        tgt = self.tgt_match.match(target)
        if not tgt:
            await ctx.send(
                "Invalid target format ({}).".format(target)
                + " Expected format like 200KB"
            )
            return

        tgt_size = unconvert_bytesize(*tgt.groups())
        file = None
        if (
            ctx.message.attachments is not None
            and len(ctx.message.attachments) > 0
        ):
            file = await ctx.message.attachments[0].read()
        elif url is not None:
            async with self._session.get(url) as resp:
                if resp.status == 200:
                    file = await resp.read()
        else:
            ctx.send("Missing file as either attachment or URL.")

        size = len(file)
        if size <= tgt_size:
            await ctx.send("Image already meets target.")
            return

        ratio = size / tgt_size

        buffer = np.frombuffer(file, dtype=np.uint8)
        img = cv2.imdecode(buffer, flags=1)

        width = int(img.shape[1] * ratio)
        height = int(img.shape[0] * ratio)

        new_img = cv2.resize(img, (width, height))
        buffer = cv2.imencode(".png", new_img)[1].tobytes()
        bufio = BytesIO(buffer)
        await ctx.send(file=File(bufio))


def setup(bot):
    bot.add_cog(ImageCog(bot))
